@@ -38,7 +38,7 @@ static const struct pci_device_id efa_pci_tbl[] = {
 };
 
 #define DRV_MODULE_VER_MAJOR           2
-#define DRV_MODULE_VER_MINOR           15
+#define DRV_MODULE_VER_MINOR           17
 #define DRV_MODULE_VER_SUBMINOR        3
 
 #ifndef DRV_MODULE_VERSION
@@ -490,6 +490,9 @@ static const struct ib_device_ops efa_dev_ops = {
 #else
 	.create_cq = efa_kzalloc_cq,
 #endif
+#ifdef HAVE_CREATE_CQ_UMEM
+	.create_cq_umem = efa_create_cq_umem,
+#endif
 #ifdef HAVE_QP_CORE_ALLOCATION
 	.create_qp = efa_create_qp,
 #else
