@@ -707,7 +707,7 @@ static void efa_qp_terminate(struct efa_qp *qp)
 	efa_cq_dec_ref_cnt(send_cq, qp->sq.wq.sub_cq_idx);
 	efa_cq_dec_ref_cnt(recv_cq, qp->rq.wq.sub_cq_idx);
 
-	dev->qp_table[qp->ibqp.qp_num] = NULL;
+	dev->qp_table[qp->ibqp.qp_num & dev->qp_table_mask] = NULL;
 
 	efa_unlock_cqs(send_cq, recv_cq);
 	spin_unlock(&dev->qp_table_lock);
@@ -1193,7 +1193,7 @@ static int efa_create_qp_kernel(struct ib_qp *ibqp, struct ib_qp_init_attr *init
 	efa_unlock_cqs(send_cq, recv_cq);
 
 	spin_lock(&dev->qp_table_lock);
-	dev->qp_table[ibqp->qp_num] = qp;
+	dev->qp_table[ibqp->qp_num & dev->qp_table_mask] = qp;
 	spin_unlock(&dev->qp_table_lock);
 
 	ibdev_dbg(&dev->ibdev, "Created qp[%d] type %d\n", qp->ibqp.qp_num, init_attr->qp_type);
@@ -1880,11 +1880,7 @@ int efa_create_cq_umem(struct ib_cq *ibcq, const struct ib_cq_init_attr *attr,
 #else
 		       struct ib_umem *umem, struct uverbs_attr_bundle *attrs)
 {
-#ifndef HAVE_EFA_KVERBS
 	struct ib_udata *udata = &attrs->driver_udata;
-#else
-	struct ib_udata *udata = NULL;
-#endif
 #endif
 #ifdef HAVE_UDATA_TO_DRV_CONTEXT
 	struct efa_ucontext *ucontext = rdma_udata_to_drv_context(
@@ -1916,18 +1912,6 @@ int efa_create_cq_umem(struct ib_cq *ibcq, const struct ib_cq_init_attr *attr,
 		goto err_out;
 	}
 
-#ifdef HAVE_EFA_KVERBS
-#ifdef HAVE_CREATE_CQ_BUNDLE
-	if (attrs) {
-		udata = &attrs->driver_udata;
-		ucontext = rdma_udata_to_drv_context(udata, struct efa_ucontext, ibucontext);
-	}
-#endif
-
-	if (!udata)
-		return efa_create_cq_kernel(ibcq, attr);
-#endif
-
 	if (offsetofend(typeof(cmd), num_sub_cqs) > udata->inlen) {
 		ibdev_dbg(ibdev,
 			  "Incompatible ABI params, no input udata\n");
@@ -1982,13 +1966,13 @@ int efa_create_cq_umem(struct ib_cq *ibcq, const struct ib_cq_init_attr *attr,
 		if (umem->length < cq->size) {
 			ibdev_dbg(&dev->ibdev, "External memory too small\n");
 			err = -EINVAL;
-			goto err_free_mem;
+			goto err_out;
 		}
 
 		if (!ib_umem_is_contiguous(umem)) {
 			ibdev_dbg(&dev->ibdev, "Non contiguous CQ unsupported\n");
 			err = -EINVAL;
-			goto err_free_mem;
+			goto err_out;
 		}
 
 		cq->cpu_addr = NULL;
@@ -2017,7 +2001,7 @@ int efa_create_cq_umem(struct ib_cq *ibcq, const struct ib_cq_init_attr *attr,
 
 	err = efa_com_create_cq(&dev->edev, &params, &result);
 	if (err)
-		goto err_free_mem;
+		goto err_free_mapped;
 
 	resp.db_off = result.db_off;
 	resp.cq_idx = result.cq_idx;
@@ -2073,11 +2057,10 @@ err_remove_mmap:
 	efa_cq_user_mmap_entries_remove(cq);
 err_destroy_cq:
 	efa_destroy_cq_idx(dev, cq->cq_idx);
-err_free_mem:
-	if (umem)
-		ib_umem_release(umem);
-	else
-		efa_free_mapped(dev, cq->cpu_addr, cq->dma_addr, cq->size, DMA_FROM_DEVICE);
+err_free_mapped:
+	if (!umem)
+		efa_free_mapped(dev, cq->cpu_addr, cq->dma_addr, cq->size,
+				DMA_FROM_DEVICE);
 
 err_out:
 	atomic64_inc(&dev->stats.create_cq_err);
@@ -2092,7 +2075,7 @@ static int efa_create_cq_umem_backport(struct ib_cq *ibcq, const struct ib_cq_in
 #else
 					struct ib_udata *udata)
 {
-	struct uverbs_attr_bundle *attrs = NULL;
+	struct uverbs_attr_bundle *attrs = rdma_udata_to_uverbs_attr_bundle(udata);
 #endif
 	struct ib_device *ibdev = ibcq->device;
 	struct ib_umem_dmabuf *umem_dmabuf;
@@ -2103,14 +2086,6 @@ static int efa_create_cq_umem_backport(struct ib_cq *ibcq, const struct ib_cq_in
 	int buffer_fd;
 	int ret;
 
-#ifndef HAVE_CREATE_CQ_BUNDLE
-	if (udata)
-		attrs = rdma_udata_to_uverbs_attr_bundle(udata);
-#endif
-
-	if (!attrs)
-		return efa_create_cq_kernel(ibcq, attr);
-
 	if (uverbs_attr_is_valid(attrs, UVERBS_ATTR_CREATE_CQ_BUFFER_VA)) {
 		ret = uverbs_copy_from(&buffer_va, attrs, UVERBS_ATTR_CREATE_CQ_BUFFER_VA);
 		if (ret)
@@ -2157,10 +2132,14 @@ static int efa_create_cq_umem_backport(struct ib_cq *ibcq, const struct ib_cq_in
 	}
 
 #ifdef HAVE_CREATE_CQ_BUNDLE
-	return efa_create_cq_umem(ibcq, attr, umem, attrs);
+	ret = efa_create_cq_umem(ibcq, attr, umem, attrs);
 #else
-	return efa_create_cq_umem(ibcq, attr, umem, udata);
+	ret = efa_create_cq_umem(ibcq, attr, umem, udata);
 #endif
+	if (ret)
+		ib_umem_release(umem);
+
+	return ret;
 }
 #endif
 
@@ -2168,6 +2147,10 @@ static int efa_create_cq_umem_backport(struct ib_cq *ibcq, const struct ib_cq_in
 int efa_create_cq(struct ib_cq *ibcq, const struct ib_cq_init_attr *attr,
 		  struct uverbs_attr_bundle *attrs)
 {
+#ifdef HAVE_EFA_KVERBS
+	if (!attrs)
+		return efa_create_cq_kernel(ibcq, attr);
+#endif
 #if !defined(HAVE_CREATE_CQ_UMEM) && defined(HAVE_UVERBS_ATTR_RAW_FD) && defined(HAVE_IB_UMEM_DMABUF_PINNED)
 	return efa_create_cq_umem_backport(ibcq, attr, attrs);
 #else
@@ -2178,6 +2161,10 @@ int efa_create_cq(struct ib_cq *ibcq, const struct ib_cq_init_attr *attr,
 int efa_create_cq(struct ib_cq *ibcq, const struct ib_cq_init_attr *attr,
 		  struct ib_udata *udata)
 {
+#ifdef HAVE_EFA_KVERBS
+	if (!udata)
+		return efa_create_cq_kernel(ibcq, attr);
+#endif
 #if !defined(HAVE_CREATE_CQ_UMEM) && defined(HAVE_UVERBS_ATTR_RAW_FD) && defined(HAVE_IB_UMEM_DMABUF_PINNED)
 	return efa_create_cq_umem_backport(ibcq, attr, udata);
 #else
