@@ -27,13 +27,19 @@
 #include <uapi/linux/bpf.h>
 #endif
 #include <linux/u64_stats_sync.h>
+#ifdef ENA_HAS_DEVLINK_HEADERS
+#include <net/devlink.h>
+#endif /* ENA_HAS_DEVLINK_HEADERS */
+#ifdef ENA_PAGE_POOL_SUPPORT
+#include <net/page_pool/helpers.h>
+#endif /* ENA_PAGE_POOL_SUPPORT */
 
 #include "ena_com.h"
 #include "ena_eth_com.h"
 
 #define DRV_MODULE_GEN_MAJOR	2
-#define DRV_MODULE_GEN_MINOR	15
-#define DRV_MODULE_GEN_SUBMINOR	0
+#define DRV_MODULE_GEN_MINOR	16
+#define DRV_MODULE_GEN_SUBMINOR	1
 
 #define DRV_MODULE_NAME		"ena"
 #ifndef DRV_MODULE_GENERATION
@@ -125,8 +131,10 @@
 
 #define ENA_MMIO_DISABLE_REG_READ	BIT(0)
 
+#ifdef ENA_LPC_SUPPORT
 struct ena_page_cache;
 
+#endif /* ENA_LPC_SUPPORT */
 #ifdef ENA_PHC_SUPPORT
 struct ena_phc_info;
 
@@ -143,6 +151,7 @@ struct ena_irq {
 struct ena_napi {
 	unsigned long last_intr_jiffies ____cacheline_aligned;
 	u8 interrupts_masked;
+	bool lost_interrupt_unmask_handled;
 	struct napi_struct napi;
 	struct ena_ring *tx_ring;
 	struct ena_ring *rx_ring;
@@ -173,8 +182,8 @@ struct ena_tx_buffer {
 	/* Indicate if bufs[0] map the linear data of the skb. */
 	u8 map_linear_data;
 
-	/* Used for detect missing tx packets to limit the number of prints */
-	u8 print_once;
+	/* Used for detecting missed tx packets */
+	bool timed_out;
 #ifdef ENA_AF_XDP_SUPPORT
 
 	/* used for ordering TX completions when needed (e.g. AF_XDP) */
@@ -212,7 +221,15 @@ struct ena_rx_buffer {
 	u32 page_offset;
 	u32 buf_offset;
 	struct ena_com_buf ena_buf;
+#ifdef ENA_LPC_SUPPORT
 	bool is_lpc_page;
+#endif /* ENA_LPC_SUPPORT */
+#ifdef ENA_PAGE_POOL_SUPPORT
+	/* Used to locally frag a page allocated from page pool via the DRB
+	 * mechanism, thus avoiding atomic ops to update the page ref.
+	 */
+	long pagecnt_bias;
+#endif /* ENA_PAGE_POOL_SUPPORT */
 } ____cacheline_aligned;
 
 struct ena_stats_tx {
@@ -230,8 +247,10 @@ struct ena_stats_tx {
 	u64 bad_req_id;
 	u64 llq_buffer_copy;
 	u64 missed_tx;
+	atomic64_t pending_timedout_pkts;
 	u64 unmask_interrupt;
 	u64 last_napi_jiffies;
+	u64 lost_interrupt;
 #ifdef ENA_AF_XDP_SUPPORT
 	u64 xsk_cnt;
 	u64 xsk_bytes;
@@ -271,9 +290,26 @@ struct ena_stats_rx {
 	u64 xdp_invalid;
 	u64 xdp_redirect;
 #endif
+#ifdef ENA_LPC_SUPPORT
 	u64 lpc_warm_up;
 	u64 lpc_full;
 	u64 lpc_wrong_numa;
+#endif /* ENA_LPC_SUPPORT */
+#ifdef ENA_PAGE_POOL_SUPPORT
+#ifdef CONFIG_PAGE_POOL_STATS
+	u64 pp_alloc_fast;
+	u64 pp_alloc_slow;
+	u64 pp_alloc_slow_hi_ord;
+	u64 pp_alloc_empty;
+	u64 pp_alloc_refill;
+	u64 pp_alloc_waive;
+	u64 pp_cached;
+	u64 pp_cache_full;
+	u64 pp_ring;
+	u64 pp_ring_full;
+	u64 pp_released_ref;
+#endif /* CONFIG_PAGE_POOL_STATS */
+#endif /* ENA_PAGE_POOL_SUPPORT */
 #ifdef ENA_AF_XDP_SUPPORT
 	u64 xsk_need_wakeup_set;
 	u64 zc_queue_pkt_copy;
@@ -296,7 +332,12 @@ struct ena_ring {
 	struct pci_dev *pdev;
 	struct napi_struct *napi;
 	struct net_device *netdev;
+#ifdef ENA_LPC_SUPPORT
 	struct ena_page_cache *page_cache;
+#endif /* ENA_LPC_SUPPORT */
+#ifdef ENA_PAGE_POOL_SUPPORT
+	struct page_pool *page_pool;
+#endif /* ENA_PAGE_POOL_SUPPORT */
 	struct ena_com_dev *ena_dev;
 	struct ena_adapter *adapter;
 	struct ena_com_io_cq *ena_com_io_cq;
@@ -306,6 +347,10 @@ struct ena_ring {
 #ifdef ENA_XDP_MB_SUPPORT
 	bool xdp_prog_support_frags;
 #endif /* ENA_XDP_MB_SUPPORT */
+#endif /* ENA_XDP_SUPPORT */
+	unsigned long last_checked_last_napi_jiffies;
+	bool last_checked_is_cq_empty;
+#ifdef ENA_XDP_SUPPORT
 	struct xdp_rxq_info xdp_rxq;
 	spinlock_t xdp_tx_lock;	/* synchronize XDP TX/Redirect traffic */
 	/* Used for rx queues only to point to the xdp tx ring, to
@@ -370,7 +415,14 @@ enum ena_busy_poll_state_t {
 	ENA_BP_STATE_POLL,
 	ENA_BP_STATE_DISABLE
 };
+
 #endif
+struct ena_keep_alive_stats {
+	u64 rx_drops;
+	u64 tx_drops;
+	u64 rx_overruns;
+};
+
 struct ena_stats_dev {
 	u64 tx_timeout;
 	u64 suspend;
@@ -379,9 +431,6 @@ struct ena_stats_dev {
 	u64 interface_up;
 	u64 interface_down;
 	u64 admin_q_pause;
-	u64 rx_drops;
-	u64 tx_drops;
-	u64 rx_overruns;
 	u64 reset_fail;
 	u64 total_resets;
 	u64 bad_tx_req_id;
@@ -398,6 +447,7 @@ struct ena_stats_dev {
 	u64 admin_to;
 	u64 device_request_reset;
 	u64 missing_first_intr;
+	struct ena_keep_alive_stats ka_stats;
 };
 
 enum ena_flags_t {
@@ -442,11 +492,13 @@ struct ena_adapter {
 	u32 num_io_queues;
 	u32 max_num_io_queues;
 
+#ifdef ENA_LPC_SUPPORT
 	/* Local page cache size when it's enabled */
 	u32 configured_lpc_size;
 	/* Current Local page cache size */
 	u32 used_lpc_size;
 
+#endif /* ENA_LPC_SUPPORT */
 #if LINUX_VERSION_CODE < KERNEL_VERSION(4, 8, 0)
 	struct msix_entry *msix_entries;
 #endif
@@ -521,6 +573,16 @@ struct ena_adapter {
 	u32 xdp_num_queues;
 
 	struct hw_timestamp_state hw_ts_state;
+	struct ena_keep_alive_stats persistent_ka_stats;
+
+	struct devlink *devlink;
+#ifdef ENA_DEVLINK_SUPPORT
+	struct devlink_port devlink_port;
+#endif /* ENA_DEVLINK_SUPPORT */
+#ifdef CONFIG_DEBUG_FS
+
+	struct dentry *debugfs_base;
+#endif /* CONFIG_DEBUG_FS */
 };
 
 #define ENA_RESET_STATS_ENTRY(reset_reason, stat) \
@@ -560,8 +622,10 @@ void ena_dump_stats_to_buf(struct ena_adapter *adapter, u8 *buf);
 
 struct sk_buff *ena_alloc_skb(struct ena_ring *rx_ring, void *first_frag, u16 len);
 
+#ifdef ENA_LPC_SUPPORT
 int ena_set_lpc_state(struct ena_adapter *adapter, bool enabled);
 
+#endif /* ENA_LPC_SUPPORT */
 int ena_update_queue_params(struct ena_adapter *adapter,
 			    u32 new_tx_size,
 			    u32 new_rx_size,
@@ -707,6 +771,7 @@ static inline int ena_tx_map_frags(struct skb_shared_info *sh_info,
 	return 0;
 }
 
+#ifdef ENA_LPC_SUPPORT
 /* Allocate a page and DMA map it
  * @rx_ring: The IO queue pair which requests the allocation
  *
@@ -715,6 +780,7 @@ static inline int ena_tx_map_frags(struct skb_shared_info *sh_info,
  */
 struct page *ena_alloc_map_page(struct ena_ring *rx_ring, dma_addr_t *dma);
 
+#endif /* ENA_LPC_SUPPORT */
 int ena_destroy_device(struct ena_adapter *adapter, bool graceful);
 int ena_restore_device(struct ena_adapter *adapter);
 void ena_get_and_dump_head_tx_cdesc(struct ena_com_io_cq *io_cq);
@@ -741,7 +807,9 @@ void ena_init_io_rings(struct ena_adapter *adapter,
 		       int first_index, int count);
 void ena_down(struct ena_adapter *adapter);
 int ena_up(struct ena_adapter *adapter);
-void ena_unmask_interrupt(struct ena_ring *tx_ring, struct ena_ring *rx_ring);
+void ena_unmask_interrupt(struct ena_ring *tx_ring,
+			  struct ena_ring *rx_ring,
+			  bool lost_interrupt);
 void ena_update_ring_numa_node(struct ena_ring *rx_ring);
 void ena_unmap_rx_buff_attrs(struct ena_ring *rx_ring,
 			     struct ena_rx_buffer *rx_info,
@@ -796,9 +864,14 @@ static inline void ena_rx_release_packet_buffers(struct ena_ring *rx_ring,
 	for (i = first_to_release; i <= last_to_release; i++) {
 		int req_id = rx_ring->ena_bufs[i].req_id;
 
+#ifndef ENA_PAGE_POOL_SUPPORT
+		/* The page pool mechanism unmaps page when the page is freed
+		 * from the pool.
+		 */
 		ena_unmap_rx_buff_attrs(rx_ring,
 					&rx_ring->rx_buffer_info[req_id],
 					ENA_DMA_ATTR_SKIP_CPU_SYNC);
+#endif /* ENA_PAGE_POOL_SUPPORT */
 		rx_ring->rx_buffer_info[req_id].page = NULL;
 	}
 }
